/*
  main.cpp - Main loop for Arduino sketches
  Copyright (c) 2005-2013 Arduino Team.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Arduino.h>
#include <SPI.h>
#include <SD.h>

// Declared weak in Arduino.h to allow user redefinitions.
int atexit(void (* /*func*/ )()) { return 0; }

// Weak empty variant initialization function.
// May be redefined by variant files.
void initVariant() __attribute__((weak));
void initVariant() { }

void setupUSB() __attribute__((weak));
void setupUSB() { }

///////////////////////////////////////////////////////////////////////////////////////////////////////
#define command_write                   1   /* Commande: stockage les fichiers à la carte SD */
#define command_read                    2   /* Commande: lire les fichiers dans la carte SD */
#define command_query                   3   /* Commande: demander les informations de la carte SD */
#define command_delete                  4   /* Commande: supprimer les fichiers dans la carte SD */
#define state_command                   1
#define state_configure                 2
#define state_txrx_start                4
#define state_txrx_end                  8
#define state_command_now               1
#define state_configure_now             3
#define state_txrx_start_now            7
#define state_txrx_end_now              0xf
File myFile;
unsigned char TxT_order;
String data_txtname = "";

/////////////////////////////////////////////////
// Fonction
// INPUT：
// OUTPUT：
////////////////////////////////////////////////
void printDirectory(File dir, int numTabs) {
  while (true) {
    File entry =  dir.openNextFile();
    if (! entry) {
      // no more files
      break;
    }
    for (uint8_t i = 0; i < numTabs; i++) {
      Serial.print('\t');
    }
    Serial.print(entry.name());
    if (entry.isDirectory()) {
      Serial.println("/");
      printDirectory(entry, numTabs + 1);
    } else {
      // files have sizes, directories do not
      Serial.print("\t\t");
      Serial.println(entry.size(), DEC);
    }
    entry.close();
  }
}
/////////////////////////////////////////////////
// Fonction de lire l'information de la serie   Identification de commande & Identification de configuration & Indentidication de debut de l'écriture de fichier
// INPUT： NULL
// OUTPUT： lire commande reussi retourner a data_commd
//          lire configuration reussi retourner a TxT_order state_configure_now
//          lire pas reussi retourner a 0
////////////////////////////////////////////////
unsigned char TxT_data_dealy(void) {                        /* Attendre des données en série  commande et configuration */
  unsigned short Timer_delay1, Timer_delay2;
  unsigned char data_commd, TxT_flag;
  String data_cache;

  while (Serial.available() <= 0) {
    Timer_delay1++;
    if (Timer_delay1 > 10000) {
      Timer_delay1 = 0;
      Timer_delay2++;
      if (Timer_delay2 >1000){
         Serial.println("Aucune information n'a ete recue");
         return 0;
      }
    }
  }
  while (Serial.available() > 0){
    if (0 == TxT_order){
      data_commd = Serial.read();
      delay(2);
      data_commd -= 48;
      return data_commd;
    }
    else if (state_command_now == TxT_order) {
      data_txtname += char(Serial.read());
      delay(2);
      TxT_flag = 1;
    }
    else if  (state_txrx_start_now == TxT_order) {
      data_cache = char(Serial.read());
      Serial.print(data_cache);
      delay(2);
      myFile.print(data_cache);
      delay(2);
      TxT_flag = 2;
    }
  }
  if (1 == TxT_flag) {
     Serial.print("Fichier d'entree est:");
     Serial.println(data_txtname);
     TxT_order |= state_configure;
     return TxT_order;
  }
  else if (2 == TxT_flag) {
     TxT_order |= state_txrx_end_now;
     Serial.println(" ");
     return TxT_order;
  }
  else {
    Serial.println("N'a pas recu d'entree efficace");
    return 0;
  }
}
/////////////////////////////////////////////////
// Les fonctions de ecrire
// INPUT： NULL
// OUTPUT： ecrire fini TxT_order state_txrx_end_now
//          ecrire pas reussi 0
////////////////////////////////////////////////
unsigned char TxT_fun_write(void) {
  TxT_order |= state_txrx_start;
  myFile = SD.open(data_txtname, FILE_WRITE);
  if (myFile) {
    if (state_txrx_end_now == TxT_data_dealy()) {
      Serial.println("Fichier ecrire fini");
      TxT_order |= state_txrx_end;
    }
    else {
      Serial.println("L'ecriture de fichier a echoue");
      TxT_order = 0;
    }
  } else {
    Serial.println("Ouvre le fichier test.txt a echoue");
    TxT_order = 0;
  }
  myFile.close();

  return TxT_order;
}
/////////////////////////////////////////////////
// Fonction de lire
// INPUT： NULL
// OUTPUT：  lire fini TxT_order state_txrx_end_now
//            lire pas reussi 0
////////////////////////////////////////////////
unsigned char TxT_fun_read(void) {
  File dataFile = SD.open(data_txtname);
  TxT_order |= state_txrx_start;
  if (dataFile) {
    while (dataFile.available()) {
      Serial.write(dataFile.read());
    }
    dataFile.close();
    Serial.println(" ");
  }
  else {
    Serial.println("Ouvre le fichier a echoue");
    return 0;
  }
  TxT_order |= state_txrx_end;
  return TxT_order;
}
/////////////////////////////////////////////////
// Fonction de supprimer
// INPUT： NULL
// OUTPUT： supprimer fini TxT_order state_txrx_end_now
//          supprimer pas reussi 0
////////////////////////////////////////////////
unsigned char TxT_fun_delete(void) {
  TxT_order |= state_txrx_start;
  if (SD.exists(data_txtname)) {
    Serial.println("exister.");
  } else {
    Serial.println("pas exister.");
    return 0;
  }
  Serial.println("En train de supprimer...");
  SD.remove(data_txtname);
  if (SD.exists(data_txtname)) {
    Serial.println("Suppression echec");
    return 0;
  } else {
    Serial.println("Suppression reussi");
  }
  TxT_order |= state_txrx_end;
  return TxT_order;
}
/////////////////////////////////////////////////
// Fonction de demander l'information de la carte SD
// INPUT： NULL
// OUTPUT：  fini TxT_order state_txrx_end_now
//           pas reussi 0
////////////////////////////////////////////////
unsigned char TxT_fun_query(void) {
  File root2;
  TxT_order |= state_txrx_start;
  root2 = SD.open("/");
  printDirectory(root2, 0);
  Serial.println("done");
  root2.close();
  TxT_order |= state_txrx_end;
  return TxT_order;
}
/////////////////////////////////////////////////
// Fonctions de traitement de la carte SD
// INPUT： LES COMMANDES
// OUTPUT： NULL
////////////////////////////////////////////////
void TxT_fun_process(unsigned char cmd){
  if (0 == TxT_order){                                                              /* Programme libre */
    TxT_order |= state_command;                                                     /* signe de commande a change pour l'etat de deja reçu le commande */
    switch(cmd) {
      case command_write:                                                           /* commande d'ecrire  */
        Serial.println("Saisissez le nom du fichier que vous voulez écrire et avec le format de. txt à la fin. (exemple: text.txt)");
        if (state_configure_now != TxT_data_dealy())break;                          /* essayer d'obtenir le nom de fichier(.txt) */
        Serial.println("Start file transfer");
        TxT_order = TxT_fun_write();                                               /* Le fonctionnement de la fonction ecrire, si reussi, retourner a l'identification end; si pas reussi, retourner a 0 */
        break;
      case command_read:                                                            /* commande de lire */
        Serial.println("Saisissez le nom du fichier que vous voulez lire et avec le format de. txt à la fin. (exemple: text.txt)");
        if (state_configure_now != TxT_data_dealy())break;
        Serial.println("Start file transfer");
        TxT_order = TxT_fun_read();                                                /* Le fonctionnement de la fonction lire, si reussi, retourner a l'identification end; si pas reussi, retourner a 0 */
        break;
      case command_query:
        TxT_order |= state_configure;                                              /* Sans étape de configuration pour demander d'informations, changer l'identification d'état directement */
        Serial.println("voici l'information de la carte SD");
        TxT_order = TxT_fun_query();                                               /* Le fonctionnement de la fonction demander d'informations, si reussi, retourner a l'identification end; si pas reussi, retourner a 0 */
        break;
      case command_delete:                                                          /* commande de supprimer */
        Serial.println("Saisissez le nom du fichier que vous voulez supprimer et avec le format de. txt à la fin. (example: text.txt)");
        if (state_configure_now != TxT_data_dealy())break;
        Serial.println("Commencer à supprimer le fichier");
        TxT_order = TxT_fun_delete();                                              /* Le fonctionnement de la fonction supprimer, si reussi, retourner a l'identification end; si pas reussi, retourner a 0 */
        break;
    }
    if (state_txrx_end_now == TxT_order){                                           /* Juger l'identification d'étate, imprimer le résultat a l'ecran */
      Serial.println("Reussi!");
      data_txtname = "";
      TxT_order = 0;
    }
    else {
      data_txtname = "";
      TxT_order = 0;
      Serial.println("pas reussi, veuillez recommencer.");
    }
  }
}

/////////////////////////////////////////////////
// ONLY ONCE
// INPUT： NULL
// OUTPUT： NULL
////////////////////////////////////////////////
void setup() {
  Serial.begin(9600);
  while (!Serial) {                                          /* L'initialisation de la série */
   ;
  }
  Serial.print("L'initialisation de la carte SD...");                  /* L'initialisation de la carte SD */
  if (!SD.begin(8)) {
    Serial.println("Carte pas reussi ou pas presenter");
    while (1);
  }
  Serial.println("L'initialisation fini");
}
/////////////////////////////////////////////////
// LOOP
// INPUT： NULL
// OUTPUT： NULL
////////////////////////////////////////////////
void loop() {
  Serial.println("ecrire input 1;  lire input 2;  information input 3;  supprimer input 4");
  TxT_fun_process(TxT_data_dealy());
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
int main(void)
{
  init();

  initVariant();

#if defined(USBCON)
  USBDevice.attach();
#endif

  setup();

  for (;;) {
    loop();
    if (serialEventRun) serialEventRun();
  }

  return 0;
}

